/*
 * ASN.1 Teletexer, a TeletexString to Unicode converter.
 * (C) 2011 Sean Leonard of SeanTek(R) <dev+asn1teletexer@seantek.com>.
 * For licensing information, consult the LICENSE.txt or the author.
 *
 * version 0.7, 2011-07-27
 */

#ifndef _ASN1TELETEXER_H_
#define _ASN1TELETEXER_H_

#include <stddef.h>
/* available as of VS 2010 on Windows; in most other ISO C-conformant environments */
#include <stdint.h>

/* Converts TeletexString-encoded data to Unicode, in UCS-2 (16-bit) units.
   All character sets supported by TeletexString convert to characters in the BMP,
	 so this function is guaranteed not to emit (and not to need to emit) surrogate code points.
	 The caller is required to allocate memory for the return buffer, which should be the same size
	 (in respective units) as the length of ttbuf. This function neither expects nor guarantees
	 NULL-termination.
	 This function allocates no heap memory; it is entirely stack-based.
	 This function initializes the ISO 2022 state machine basically as described in X.690, adding
	 GR=G1 and G1=ISO 8859-1 (RH Latin-1) so that the string will initially be interpreted as
	 ISO 8859-1 data until a control function alters the state.
	 @param ttbuf: A pointer to the octets (bytes) of the actual TeletexString data. Must be provided.
	 @param len: The length of the ttbuf and retbuf buffers, in their respective units. Can be 0.
	 @param retbuf: A pointer to the array that will receive the Unicode output. The array must be allocated
	        to at least the same length (in respective units) as ttbuf.
	 @return The number of Unicode units written to retbuf. This number will never be greater than len.
	 Errors:
	 If a non-fatal error occurs, the function writes U+FFFD REPLACEMENT CHARACTER to the buffer and continues.
	 This kind of error usually happens when a code point does not have a Unicode mapping.
	 The code continues decoding.
	 A fatal error is an error where the function cannot continue, such as when an invalid escape sequence
	 is found. Because the escape sequence is unknown, the decoder cannot know how to decode the subsequent
	 octets (bytes). If a fatal error occurs, the function writes U+FFFF <not a character> to the output,
	 and quits.
	 Both cases are non-fatal for display purposes but are fatal for X.500 DirectoryString comparison purposes.
	 The string preparation rules in X.520 and RFC 4518 will cause the comparison to fail because
	 U+FFFD and U+FFFF are prohibited.
 */
size_t TeletexString_to_ucs2(const uint8_t *ttbuf, size_t len, uint16_t *retbuf);

/* Converts TeletexString-encoded data to Unicode, in UCS-4 (32-bit) units.
   See TeletexString_to_ucs2 for complete notes.
 */
size_t TeletexString_to_ucs4(const uint8_t *ttbuf, size_t len, uint32_t *retbuf);

/* Converts TeletexString-encoded data to Unicode, in wchar_t units.
   See TeletexString_to_ucs2 for complete notes.
 */
size_t TeletexString_to_wcs(const uint8_t *ttbuf, size_t len, wchar_t *retbuf);

#endif
