/*
 * DBCS Generator for ASN.1 Teletexer.
 * (C) 2011 Sean Leonard of SeanTek(R) <dev+asn1teletexer@seantek.com>.
 * For licensing information, consult the LICENSE.txt or the author.
 *
 * version 0.7, 2011-07-27
 *
 * This program executes unit tests against the ASN.1 Teletexer code.
 * Specifically, it outputs 87.txt and 58.txt, which enumerate the entire 94x94 grid of
 * possible Japanese and Chinese characters. The output should be compared against the
 * expected results, which are recorded permanently in 87-official.txt and 58-official.txt.
 * This program outputs in UTF-16LE (little endian).
 */

#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS
#endif

#include <stdio.h>
#include <memory.h>
#include <string.h>
#include <wchar.h>
#include <assert.h>

#include "../asn1teletexer.h"

static uint8_t dbcstbl[60000] = {0x1B};

static uint16_t unitbl[60000] = {0xFEFF};

int main(int argc, const char * argv[]) {
	size_t i = 3, j, k, ret, out87, out58;
	FILE *fp87 = NULL, *fp58 = NULL;
	const char *name87, *name58;
	int reverse = 0;
	if (argc < 2) {
		name87 = "87.txt";
		name58 = "58.txt";
	} else if (argc == 2) {
		puts("Need two file names. First file name is for #87; second is for #58.");
		return 1;
	} else {
		name87 = argv[1];
		name58 = argv[2];
	}

	dbcstbl[1] = 0x24;
	dbcstbl[2] = 0x42; /* #87 */
	
	for (j = 1; j < 95; j++) {
		for (k = 1; k < 95; k++) {
			dbcstbl[i++] = (uint8_t)j + 0x20;
			dbcstbl[i++] = (uint8_t)k + 0x20;
			if (k != 94) dbcstbl[i++] = '\t';
		}
		/* use CP/M / DOS / e-mail style */
		dbcstbl[i++] = '\r';
		dbcstbl[i++] = '\n';
	}

	if (((uint8_t*)unitbl)[0] == 0xFE && ((uint8_t*)unitbl)[1] == 0xFF) {
		unitbl[0] = 0xFFFE;
		reverse = 1;
	}

	ret = TeletexString_to_ucs2(dbcstbl, i, unitbl + 1);
	fp87 = fopen(name87, "wb");
	assert(fp87);

	if (reverse) for (j = 1; j <= ret; j++) {
		unitbl[j] = (unitbl[j] >> 8) | (unitbl[j] << 8); /* byte swap */
	}

	out87 = fwrite(unitbl, 1, sizeof(uint16_t)*(ret + 1), fp87);
	fclose(fp87);
	fp87 = NULL;
	
	dbcstbl[2] = 0x41; /* #58 */
	ret = TeletexString_to_ucs2(dbcstbl, i, unitbl + 1);
	fp58 = fopen(name58, "wb");
	assert(fp58);

	if (reverse) for (j = 1; j <= ret; j++) {
		unitbl[j] = (unitbl[j] >> 8) | (unitbl[j] << 8); /* byte swap */
	}

	out58 = fwrite(unitbl, 1, sizeof(uint16_t)*(ret + 1), fp58);
	fclose(fp58);
	fp58 = NULL;
	return 0;
}
