/*
 * ASN.1 Teletexer, a TeletexString to Unicode converter.
 * (C) 2011 Sean Leonard of SeanTek(R) <dev+asn1teletexer@seantek.com>.
 *
 * version 0.7, 2011-07-27
 *
 * moz_asn1teletexer_bindings.cpp: C++ implementation file for use with
 * asn1teletexer. When compiling asn1teletexer.c, add the following preprocessor
 * defines at the command-line:
 *  EXTERN_JISX0208_1997 JISX0208_1997_moz
 *  EXTERN_GB2312_80_GB6345_1_86 GB2312_80_GB6345_1_86_moz
 */


#include <nsISupports.h>
#include <nsCOMPtr.h>
#include <nsComponentManagerUtils.h>
#include "asn1teletexer.h"

#define NS_UNICODEDECODER_CONTRACTID_BASE "@mozilla.org/intl/unicode/decoder;1?charset="

#define NS_IUNICODEDECODER_moz192_IID	\
	{ 0x25359602, 0xfc70, 0x4d13,	\
		{ 0xa9, 0xab, 0x80, 0x86, 0xd3, 0x82, 0x7c, 0xd }}

class nsIUnicodeDecoder_moz192 : public nsISupports
{
public:
	NS_IMETHOD Convert(const char * aSrc, PRInt32 * aSrcLength, PRUnichar * aDest, PRInt32 * aDestLength) = 0;
	NS_IMETHOD GetMaxLength(const char * aSrc, PRInt32 aSrcLength, PRInt32 * aDestLength) = 0;
	NS_IMETHOD Reset() = 0;
	virtual void SetInputErrorBehavior(PRInt32 aBehavior) = 0;
	virtual PRUnichar GetCharacterForUnMapped() = 0;
};

#define NS_IUNICODEDECODER_moz191_IID	\
	{ 0xb2f178e1, 0x832a, 0x11d2,	\
		{ 0x8a, 0x8e, 0x0, 0x60, 0x8, 0x11, 0xa8, 0x36 }}

class nsIUnicodeDecoder_moz191 : public nsISupports
{
public:
	NS_IMETHOD Convert(const char * aSrc, PRInt32 * aSrcLength, PRUnichar * aDest, PRInt32 * aDestLength) = 0;
	NS_IMETHOD GetMaxLength(const char * aSrc, PRInt32 aSrcLength, PRInt32 * aDestLength) = 0;
	NS_IMETHOD Reset() = 0;
};

NS_DEFINE_IID(nsIUnicodeDecoder_moz191IID, NS_IUNICODEDECODER_moz191_IID);
NS_DEFINE_IID(nsIUnicodeDecoder_moz192IID, NS_IUNICODEDECODER_moz192_IID);

static uint16_t moz_common_convert(uint8_t bytes[2], const char *charset_contractID) {
	// dec_supports2 handles both variants of nsIUnicodeDecoder, for appropriate lifetime management.
	nsCOMPtr<nsISupports> dec_supports, dec_supports2;
	PRUnichar outchars[2] = {0};
	nsresult rv;
	PRInt32 outcharslen = 2;
	PRInt32 srclen = 2;

	// calls NS_GetComponentManager, then ->CreateInstanceByContractID
	rv = CallCreateInstance(charset_contractID, NULL, NS_GET_IID(nsISupports), getter_AddRefs(dec_supports));
	NS_ASSERTION(NS_SUCCEEDED(rv), "Could not create the Unicode decoder.");
	if (NS_FAILED(rv)) return rv;

	rv = dec_supports->QueryInterface(nsIUnicodeDecoder_moz192IID, getter_AddRefs(dec_supports2));
	if (NS_SUCCEEDED(rv))
	{
		nsIUnicodeDecoder_moz192 *dec_moz192 = (nsIUnicodeDecoder_moz192*)dec_supports2.get();
		// not necessary? dec_moz192->SetInputErrorBehavior(nsIUnicodeDecoder_moz192::kOnError_Signal);
		rv = dec_moz192->Convert((const char*)bytes, &srclen, outchars, &outcharslen);
	}
	else if (NS_SUCCEEDED(rv = dec_supports->QueryInterface(nsIUnicodeDecoder_moz191IID, getter_AddRefs(dec_supports2))))
	{
		nsIUnicodeDecoder_moz191 *dec_moz191 = (nsIUnicodeDecoder_moz191*)dec_supports2.get();
		rv = dec_moz191->Convert((const char*)bytes, &srclen, outchars, &outcharslen);
	}
	else
	{	// This will happen if the decoder version is new.
		NS_ERROR("Could not get a proper interface pointer to the Unicode decoder.");
		return 0xFFFD;
	}

	if (NS_FAILED(rv))
		return 0xFFFD;

	NS_ASSERTION(outcharslen == 1, "Only supposed to get one char out.");

	if (outchars[0] >= 0xE000 && outchars[0] <= 0xF8FF)
		return 0xFFFD; // no PUA allowed

	return (uint16_t)outchars[0];
}

NS_EXTERN_C uint16_t JISX0208_1997_moz(const uint8_t b[2]) {
	uint8_t bytes[2] = {b[0] | 0x80, b[1] | 0x80};

	if (bytes[0] == 0xA1)
	{
		if (bytes[1] == 0xA0 + 29) return 0x2014; /* EM DASH */
		if (bytes[1] == 0xA0 + 33) return 0x301C; /* WAVE DASH */
		if (bytes[1] == 0xA0 + 34) return 0x2016; /* DOUBLE VERTICAL LINE */
		if (bytes[1] == 0xA0 + 61) return 0x2212; /* MINUS SIGN */
	}
	else if (bytes[0] == 0xA0 + 13) return 0xFFFD; /* no NEC row 13 */
	else if (bytes[0] >= 0xA0 + 85) return 0xFFFD; /* no rows 89-92 */
	
	return moz_common_convert(bytes, NS_UNICODEDECODER_CONTRACTID_BASE "EUC-JP");
}

NS_EXTERN_C uint16_t GB2312_80_GB6345_1_86_moz(const uint8_t b[2]) {
	uint8_t bytes[2] = {b[0] | 0x80, b[1] | 0x80};
	
	if (bytes[0] == 0xA0 + 11) { /* half-width pinyin characters (same as row 8, for our purposes) */
		if (bytes[1] <= 0xA0 + 32) bytes[0] = 0xA0 + 8;
		else return 0xFFFD;
	}

	if (bytes[0] == 0xA0 + 8) { /* "full-width" pinyin and bopomofo */
		if (bytes[1] == 0xA0 + 28) return 0x1E3F;
	} else if (bytes[0] == 0xA0 + 10) { /* ISO646-CN */
		if (bytes[1] == 0xA0 + 4) return 0xA5; /* Yen Sign */
		else if (bytes[1] == 0xA0 + 94) return 0x203E; /* Overline */
		else return (uint16_t)(bytes[1] - 0x80);
	} else if (bytes[0] == 0xA0 + 2) {
		if (bytes[1] < 0xA0 + 17) return 0xFFFD; /* no lowercase roman numerals (02-01 through 02-10) */
		if (bytes[1] == 0xA0 + 67) return 0xFFFD; /* no euro symbol */
	}

	return moz_common_convert(bytes, NS_UNICODEDECODER_CONTRACTID_BASE "GB2312");
}
