/*
 * Tests for ASN.1 Teletexer.
 * (C) 2011 Sean Leonard of SeanTek(R) <dev+asn1teletexer@seantek.com>.
 * For licensing information, consult the LICENSE.txt or the author.
 *
 * version 0.7, 2011-07-27
 *
 * This program is a helpful standalone program that converts TeletexString data
 * to Unicode data using standard streams (stdin, stdout).
 * If successful, the program returns 0. If not successful, the program
 * returns 1 if U+FFFD was encountered, 2 if U+FFFF was encountered, and 3 if
 * both U+FFFF and U+FFFD were encountered. If out of memory, 4 is returned.
 * By default, UTF-16 with a BOM is output.
 * The output endianness is the default endianness of the platform.
 * Options:
 *  -8, --UTF-8 (/8, /UTF-8 on Win32): outputs UTF-8 without a BOM instead of UTF-16.
 */

#include "../asn1teletexer.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define BUF_SIZE 65536

int main(int argc, const char * argv[]) {
	int utf8 = 0, i, bom = 0;
	uint8_t *buf;
	size_t bufread = 0;
	size_t j;
	uint16_t *ubuf;
	size_t ubuflen;
	int ret = 0;

	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-8") || !strcmp(argv[i], "--UTF-8")) {
			utf8 = 1;
		} else if (!strcmp(argv[i], "-bom") || !strcmp(argv[i], "--BOM")) {
			bom = 1;
#ifdef WIN32
		} else if (!strcmp(argv[i], "/8") || !strcmp(argv[i], "/UTF-8")) {
			utf8 = 1;
		} else if (!strcmp(argv[i], "/bom") || !strcmp(argv[i], "/BOM")) {
			bom = 1;
#endif
		}
	}

	buf = (uint8_t*)malloc(BUF_SIZE);
	if (!buf) return 4;

	while (!feof(stdin)) {
		int c;
		size_t z;
		buf = (uint8_t*)realloc(buf, bufread + BUF_SIZE);
		if (!buf) return 4; /* memory leak of original buf, but we don't care */
		// bufread += fread(buf + bufread, 1, BUF_SIZE, stdin);
		for (z = 0; z < BUF_SIZE; z++) {
			c = getchar();
			if (c == EOF) break;
			*(buf + bufread + z) = (uint8_t)c;
		}
		bufread += z;
	}

	ubuf = (uint16_t*)malloc(bufread * sizeof(uint16_t));
	if (!ubuf) return 4;

	ubuflen = TeletexString_to_ucs2(buf, bufread, ubuf);
	if (ubuflen && ubuf[ubuflen-1] == 0xFFFF) ret |= 2;

	for (j = 0; j < ubuflen; j++) {
		if (ubuf[j] == 0xFFFD) {
			ret |= 1;
			break;
		}
	}

	if (bom) {
		if (utf8) {
			uint8_t utf8bombuf[3] = {0xEF, 0xBB, 0xBF};
			fwrite(utf8bombuf, 1, 3, stdout);
		} else {
			uint8_t bombuf[2];
			*((uint16_t*)bombuf) = (uint16_t)0xFEFF;
			fwrite(bombuf, 1, 2, stdout);
		}
	}

	if (utf8) {
		uint8_t outbuf[3];
		for (j = 0; j < ubuflen; j++) {
			if (ubuf[j] < 0x80) {
				outbuf[0] = (uint8_t)ubuf[j];
				fwrite(outbuf, 1, 1, stdout);
			} else if (ubuf[j] < 0x800) {
				outbuf[0] = 0xC0 | (ubuf[j] >> 6);
				outbuf[1] = 0x80 | (ubuf[j] & 0x3F);
				fwrite(outbuf, 1, 2, stdout);
			} else {
				outbuf[0] = 0xE0 | (ubuf[j] >> 12);
				outbuf[1] = 0x80 | ((ubuf[j] >> 6) & 0x3F);
				outbuf[2] = 0x80 | (ubuf[j] & 0x3F);
				fwrite(outbuf, 1, 3, stdout);
			}
		}
	} else fwrite(ubuf, sizeof(uint16_t), ubuflen, stdout);

	return ret;
}
